#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <libgen.h>

#include "error.h"
#include "math.h"
#include "config.h"
#include "log.h"
#include "auth.h"

char config_yes_no(char *what)
{
        if (what[0] == '1' || strcasecmp(what, "yes") == 0 || strcasecmp(what, "on") == 0)
        {
                return 1;
        }

        return 0;
}

void load_config(const char *config, config_t *pconfig)
{
	char *dummy = strdup(config);
	char *cur_dir = dirname(dummy);
        int linenr = 0;
        FILE *fh = fopen(config, "r");
        if (!fh)
                error_exit("error opening configuration file '%s'", config);

	/* set defaults */
	pconfig -> max_number_of_mem_pools = 14;
	pconfig -> max_number_of_disk_pools = 128;
	pconfig -> min_store_on_disk_n = 5;

	pconfig -> bitcount_estimator = BCE_SHANNON;

	pconfig -> listen_adapter    = (char *)"0.0.0.0";
	pconfig -> listen_port       = 55225;
	pconfig -> listen_queue_size = 5;
	pconfig -> disable_nagle     = 0;
	pconfig -> enable_keepalive  = 1;

	pconfig -> reset_counters_interval    = 60;
	pconfig -> statistics_interval        = 300;
	pconfig -> ping_interval              = 601;
	pconfig -> kernelpool_filled_interval = 3600;

	pconfig -> stats_file = NULL;

	pconfig -> communication_timeout              = 15;
	pconfig -> communication_session_timeout      = 3600; /* 0 for no timeout */
	pconfig -> default_sleep_time_when_pools_full = 10;
	pconfig -> default_sleep_when_pools_empty     = 1;
	pconfig -> default_max_sleep_when_pools_empty = 60;
	pconfig -> when_pools_full_allow_submit_interval = 15;

	pconfig -> default_max_bits_per_interval = 16000000;

	pconfig -> ignore_rngtest_fips140 = 0;
	pconfig -> ignore_rngtest_scc = 0;
	pconfig -> scc_threshold = 0.2;

	pconfig -> allow_event_entropy_addition = 1;
	pconfig -> add_entropy_even_if_all_full = 0;
	pconfig -> allow_prng = 0;

	pconfig -> auth_password = NULL;

        for(;;)
        {
		double parvald;
		int parval;
                char read_buffer[4096], *lf, *par;
                char *cmd = fgets(read_buffer, sizeof(read_buffer), fh), *is;
                if (!cmd)
                        break;
                linenr++;

                if (read_buffer[0] == '#' || read_buffer[0] == ';')
                        continue;

		while(*cmd == ' ') cmd++;

                lf = strchr(read_buffer, '\n');
                if (lf) *lf = 0x00;

		if (strlen(cmd) == 0)
			continue;

                is = strchr(read_buffer, '=');
                if (!is)
                        error_exit("invalid line at line %d: '=' missing", linenr);

                *is = 0x00;
                par = is + 1;
		while(*par == ' ') par++;
		parval = atoi(par);
		parvald = atof(par);

		is--;
		while(*is == ' ') { *is = 0x00 ; is--; }

		if (strcmp(cmd, "max_number_of_mem_pools") == 0)
			pconfig -> max_number_of_mem_pools = parval;
		else if (strcmp(cmd, "max_number_of_disk_pools") == 0)
			pconfig -> max_number_of_disk_pools = parval;
		else if (strcmp(cmd, "min_store_on_disk_n") == 0)
			pconfig -> min_store_on_disk_n = parval;
		else if (strcmp(cmd, "listen_adapter") == 0)
			pconfig -> listen_adapter = strdup(par);
		else if (strcmp(cmd, "password") == 0)
		{
			char *p_file = (char *)malloc(strlen(cur_dir) + strlen(par) + 1 + 1);
			if (par[0] == '/')
				strcpy(p_file, par);
			else
				sprintf(p_file, "%s/%s", cur_dir, par);
			dolog(LOG_INFO, "Load password from %s", p_file);
			pconfig -> auth_password = get_password_from_file(p_file);
			free(p_file);
		}
		else if (strcmp(cmd, "bitcount_estimator") == 0)
		{
			if (strcmp(par, "shannon") == 0)
				pconfig -> bitcount_estimator = BCE_SHANNON;
			else if (strcmp(par, "compression") == 0)
				pconfig -> bitcount_estimator = BCE_COMPRESSION;
			else
				error_exit("bitcount_estimator of type '%s' is not known", par);
		}
		else if (strcmp(cmd, "listen_port") == 0)
			pconfig -> listen_port = parval;
		else if (strcmp(cmd, "listen_queue_size") == 0)
			pconfig -> listen_queue_size = parval;
		else if (strcmp(cmd, "disable_nagle") == 0)
			pconfig -> disable_nagle = config_yes_no(par);
		else if (strcmp(cmd, "enable_keepalive") == 0)
			pconfig -> enable_keepalive = config_yes_no(par);
		else if (strcmp(cmd, "reset_counters_interval") == 0)
			pconfig -> reset_counters_interval = parval;
		else if (strcmp(cmd, "statistics_interval") == 0)
			pconfig -> statistics_interval = parval;
		else if (strcmp(cmd, "ping_interval") == 0)
			pconfig -> ping_interval = parval;
		else if (strcmp(cmd, "kernelpool_filled_interval") == 0)
			pconfig -> kernelpool_filled_interval = parval;
		else if (strcmp(cmd, "stats_file") == 0)
			pconfig -> stats_file = strdup(par);
		else if (strcmp(cmd, "communication_timeout") == 0)
			pconfig -> communication_timeout = parval;
		else if (strcmp(cmd, "communication_session_timeout") == 0)
			pconfig -> communication_session_timeout = parval;
		else if (strcmp(cmd, "default_sleep_time_when_pools_full") == 0)
			pconfig -> default_sleep_time_when_pools_full = parval;
		else if (strcmp(cmd, "default_sleep_when_pools_empty") == 0)
			pconfig -> default_sleep_when_pools_empty = parval;
		else if (strcmp(cmd, "default_max_sleep_when_pools_empty") == 0)
			pconfig -> default_max_sleep_when_pools_empty = parval;
		else if (strcmp(cmd, "default_max_bits_per_interval") == 0)
			pconfig -> default_max_bits_per_interval = parval;
		else if (strcmp(cmd, "ignore_rngtest_fips140") == 0)
			pconfig -> ignore_rngtest_fips140 = config_yes_no(par);
		else if (strcmp(cmd, "ignore_rngtest_scc") == 0)
			pconfig -> ignore_rngtest_scc = config_yes_no(par);
		else if (strcmp(cmd, "allow_event_entropy_addition") == 0)
			pconfig -> allow_event_entropy_addition = config_yes_no(par);
		else if (strcmp(cmd, "add_entropy_even_if_all_full") == 0)
			pconfig -> add_entropy_even_if_all_full = config_yes_no(par);
		else if (strcmp(cmd, "allow_prng") == 0)
			pconfig -> allow_prng = config_yes_no(par);
		else if (strcmp(cmd, "scc_threshold") == 0)
			pconfig -> scc_threshold = parvald;
		else if (strcmp(cmd, "when_pools_full_allow_submit_interval") == 0)
			pconfig -> when_pools_full_allow_submit_interval = parval;
		else
			error_exit("%s=%s not understood", cmd, par);
	}

	dolog(LOG_DEBUG, "read %d configuration file lines", linenr);

	fclose(fh);

	free(dummy);
}
